##############################################################################
#
#	dc.pm	-	Datacenter class
#	
#	This file is part of the EETCO tool written by 
#	Damien Hardy, Isidoros Sideris and Yiannakis Sazeides
#	University of Cyprus
#	The research leading to this tool is supported by the European Commission
#	FP7 project ”Energy-conscious 3D Server-on-Chip for Green Cloud
#	Services (Project No:247779 ”EuroCloud”)”
#
#	This source file is distributed "as is" in the hope that it will be useful.
#	The tool comes with no warranty, and no author or distributor accepts any
#	responsability for the consequences of its use.
#
#	Everyone is granted permission to copy, modify and redistribute
#	this tool set under the following conditions:
#
#		This source code is distributed for non-commercial use only.
#		Please contact the maintainer for restrictions applying to
#		commercial use.
#
#		Permission is granted to anyone to make or distribute copies
#		of this source code, either as received or modified, in any
#		medium, provided that all copyright notices, permission and
#		nonwarranty notices are preserved, and that the distributor
#		grants the recipient permission for further redistribution as
#		permitted by this document.
#
#		Permission is granted to distribute this file in compiled
#		or executable form under the same conditions that apply for
#		source code, provided that either:
#
#		A.	it is accompanied by the corresponding machine-readable
#			source code,
#		B.	it is accompanied by a written offer, with no time limit,
#			to give anyone a machine-readable copy of the corresponding
#			source code in return for reimbursement of the cost of
#			distribution.  This written offer must permit verbatim
#			duplication by anyone, or
#		C.	it is distributed by someone who received only the
#			executable form, and is accompanied by a copy of the
#			written offer of source code that they received concurrently.
#
#	In other words, you are welcome to use, share and improve this
#	source file.  You are forbidden to forbid anyone else to use, share
#	and improve what you give them.
#
##############################################################################

package dc;

use strict;
use warnings;

use resource;


#list of parameters which have to be present in a datacenter configuration file
@dc::list_parameters_name=("c_buildingpersqm","a_coolingarea","dcdepr","c_coolpowerdeplperW","c_electperKWH","PUE","salary_perrackpermonth","report_resources","n_othermodules","p_othermodules","p_othermodulesidle","n_othermodulesrepl","c_nonsrvmodule","u_othermodules","depr_othermodules","a_othermodules","n_racks_othermodules");
#condition they have to meet
%dc::input_parameters_check_values=("c_buildingpersqm","<" ,"a_coolingarea","<" ,"dcdepr","<=" ,"c_coolpowerdeplperW","<" ,"c_electperKWH","<" ,"PUE","<" ,"salary_perrackpermonth","<" ,"report_resources","<" ,"n_othermodules","<" ,"p_othermodules","<" ,"p_othermodulesidle","<" ,"n_othermodulesrepl","<" ,"c_nonsrvmodule","<" ,"u_othermodules","<" ,"depr_othermodules","<" ,"a_othermodules","<" ,"n_racks_othermodules","<" );


################################################
#
#	constructor & initialization 
#	(parsing of the input file)
#
################################################
sub new($)
{
	my ($class, $filename)= @_;
	my $self={};
	bless ($self , $class);
	
	$self->{filename}=$filename;
	$self->{srv_list}=[];
	
	if(!open FILE,"<$filename")
	{
		die "cannot open file: $filename\n";
	}
	while(defined(my $line=<FILE>))
	{
		if($line =~ /^\s*(\w+)\s+(\d+(?:.\d+)?(?:e[+|-]?\d+)?)/)
		{
			my $variable=$1;
			my $value=$2;
			
			$self->{"$variable"}=$2;
		}
	}
	close FILE;
	
	###check if all the parameters have been correctly defined
	my $error=0;
	foreach my $pname (@dc::list_parameters_name)
	{
		if(!defined($self->{"$pname"}))
		{
			print("$pname not defined\n");
			$error=1;
		}
		else
		{
			if($dc::input_parameters_check_values{"$pname"} eq "<=")
			{
				if($self->{"$pname"} <= 0)
				{
					$error=1;
					print("$pname not correctly defined\n");
				}
			}
			
			if($dc::input_parameters_check_values{"$pname"} eq "<")
			{
				if($self->{"$pname"} < 0)
				{
					$error=1;
					print("$pname not correctly defined\n");
				}
			}
		}
	}
	if($error==1)
	{
			die "Interrupt:parameters are not defined in $filename\n";
	}
	
	if($self->{report_resources} !=0 && $self->{report_resources} !=1)
	{
		print("report_resources not correctly defined\n");
		die "Interrupt:parameters are not defined in $filename\n";
	}
	##end check
	
	bless ($self , $class);
	return $self;
}




################################################
#
#	print input param
#
################################################
sub print_param()
{
	my $self=shift;
	
	print("-----------------------\nDC parameters\n-----------------------\n");
	
	foreach my $pname ( @dc::list_parameters_name ) {print("$pname = $self->{$pname}\n");}
	
	#optional parameters
	if(defined($self->{"c_elecpeak"})){print("c_elecpeak = $self->{c_elecpeak}\n");}
	if(defined($self->{"cost_of_money"})){print("cost_of_money = $self->{cost_of_money}\n");}
	
	print("-----------------------\nResources parameters\n-----------------------\n");
	
	foreach my $srv (@{$self->{srv_list}}) {$srv->print_param();}
}


################################################
#
#	accessors
#		get (variable name)
#		set (variable name,value)
#
################################################
sub get($)
{
	my ($class, $pname)= @_;
	my $self=shift;
	return $self->{"$pname"};
}

sub set($$)
{
	my ($class, $pname,$val)= @_;
	my $self=shift;
	$self->{"$pname"}=$val;
}


################################################
#
#	add a server configuration to the dc
#
################################################

sub addServerConfiguration($)
{
	my ($class,$val)= @_;
	my $self=shift;
	push (@{$self->{srv_list}}, $val);
	
	#check if the server configuration is correct
	$val->check_validity();
	
}


################################################
#
#	TCO Estimation function
#
################################################
sub TCOEstimation()
{
	my $self=shift;
	
	my $c_server=0;
	my $c_power=0;
	my $c_acquisition=0;
	my $c_maintenance=0;
	my $c_network=0;
	
	foreach my $srv ( @{$self->{srv_list}} )
	{
		$srv->sparesEstimation();
		$c_server += $srv->c_serverEstimation();
		$c_power += $srv->c_powerEstimation();
		$c_acquisition += $srv->c_acquisitionEstimation();
		$c_maintenance += $srv->c_maintenanceEstimation();
		$c_network += $srv->c_networkEstimation();
	}
	
	###add other modules
	
	#server cost
	my $c_othermodules= ($self->{n_othermodules}*$self->{c_nonsrvmodule}  ) / ( $self->{depr_othermodules}*12)  ;
	$c_server += $c_othermodules;
	
	#power cost
	my $p_total_peak_othermodules=$self->{n_othermodules}*$self->{p_othermodules};
	my $p_total_idle_othermodules=$self->{n_othermodules}*$self->{p_othermodulesidle};
	my $p_total_avg_othermodules=$self->{u_othermodules}*$p_total_peak_othermodules + (1-$self->{u_othermodules})*$p_total_idle_othermodules;
	$c_power += (($self->{PUE} * $p_total_avg_othermodules *30*24)/1000) * $self->{c_electperKWH}; #30*24/1000 -> to translate Watts in KWh per month
	
	#acquisition cost
	my $a_dc_othermodules= $self->{a_othermodules} * $self->{a_coolingarea}; #extra area for cooling
	my $c_building_acquisition_othermodules=$a_dc_othermodules*$self->{c_buildingpersqm}; #building cost
	my $c_cooling_acquisition_othermodules=$self->{c_coolpowerdeplperW}*$p_total_peak_othermodules;
	$c_acquisition += ($c_building_acquisition_othermodules+$c_cooling_acquisition_othermodules)/($self->{dcdepr}*12);
	
	#maintenance cost
	my $c_othermodulesrepl= ($self->{n_othermodulesrepl}*$self->{c_nonsrvmodule}) / ($self->{depr_othermodules}*12);
	$c_maintenance += $c_othermodulesrepl + $self->{n_racks_othermodules} * $self->{salary_perrackpermonth};
	
	##store results
	$self->{c_server}=$c_server;
	$self->{c_power}=$c_power;
	$self->{c_acquisition}=$c_acquisition;
	$self->{c_maintenance}=$c_maintenance;
	$self->{c_network}=$c_network;
	$self->{TCO}=$c_server+$c_power+$c_acquisition+$c_maintenance+$c_network;
	
	
}


################################################
#
#	enviromental impact Estimation function
#
################################################
sub environmentalImpactEstimation()
{
	my $self=shift;
	
	my $p_total_avg=0;
	foreach my $srv ( @{$self->{srv_list}} )
	{
		$p_total_avg += $srv->get("p_total_avg");
	}
	
	###add other modules
	#p_total_avg_othermodules (idem to TCO estimate)
	my $p_total_peak_othermodules=$self->{n_othermodules}*$self->{p_othermodules};
	my $p_total_idle_othermodules=$self->{n_othermodules}*$self->{p_othermodulesidle};
	my $p_total_avg_othermodules=$self->{u_othermodules}*$p_total_peak_othermodules + (1-$self->{u_othermodules})*$p_total_idle_othermodules;
	$p_total_avg += $p_total_avg_othermodules;
	
	$p_total_avg *= $self->{PUE};
	
	#compute environmental impact
	my $KWhrperyr=$p_total_avg*24*365/1000;
	my $co2peryr=$KWhrperyr*0.54522; #kg CO2
	
	##store results
	$self->{co2peryr}=$co2peryr;
	
	
}


###########################################
#
#	availability Estimation Function
#
###########################################
sub availabilityEstimation()
{	
	my $self=shift;
	foreach my $srv ( @{$self->{srv_list}} ) {$srv->availabilityEstimation();}
}


###########################################
#
#	print results Function
#
###########################################
sub print_results()
{	
	my $self=shift;
	
	print("-----------------------\nDC results\n-----------------------\n");
	
	my $tco=myround($self->{TCO});	
	my $c_acquisition=myround($self->{c_acquisition});
	my $c_power=myround($self->{c_power});
	my $c_server=myround($self->{c_server});
	my $c_maintenance=myround($self->{c_maintenance});
	my $c_network=myround($self->{c_network});
	
	print ("\ntco: $tco \$/month \n");
	
	print("\tc_infrastructure: \t$c_acquisition \$/month\n");
	print("\tc_server: \t\t$c_server \$/month\n");
	print("\tc_power: \t\t$c_power \$/month\n");
	print("\tc_network: \t\t$c_network \$/month\n");
	print("\tc_maintenance: \t\t$c_maintenance \$/month\n\n");
	
	#get info from srvs
	my $n_racks=0;
	my $a_dc=0;
	my $p_total_avg=0;
	my $p_total_peak=0;
	foreach my $srv (@{$self->{srv_list}})
	{
		$n_racks += $srv->get("n_racks");
		$a_dc += $srv->get("a_dc");
		$p_total_avg += $srv->get("p_total_avg");
		$p_total_peak += $srv->get("p_total_peak");
	}
	
	
	#add other modules
	$n_racks+=$self->{n_racks_othermodules};
	$a_dc+=$self->{a_othermodules} * $self->{a_coolingarea};
	
	my $p_total_peak_othermodules=$self->{n_othermodules}*$self->{p_othermodules};
	my $p_total_idle_othermodules=$self->{n_othermodules}*$self->{p_othermodulesidle};
	my $p_total_avg_othermodules=$self->{u_othermodules}*$p_total_peak_othermodules + (1-$self->{u_othermodules})*$p_total_idle_othermodules;
	$p_total_avg+=$p_total_avg_othermodules;
	$p_total_peak+=$p_total_peak_othermodules;
	#end other modules
	
	print("n_racks: \t$n_racks\n");

	$a_dc=myround($a_dc);
	print("a_dc: \t\t$a_dc m^2\n");

	$p_total_avg=myround($p_total_avg*$self->{PUE});
	$p_total_peak=myround($p_total_peak*$self->{PUE});
	print("p_total_avg: \t$p_total_avg W\n");
	print("p_total_peak:\t$p_total_peak W\n\n");
	
	my $co2peryr =myround($self->{co2peryr});
	print("co2peryr: \t$co2peryr kg\n");
	
	###################

	if($self->{report_resources} == 1)
	{
		print("-----------------------\nResources results\n-----------------------\n");
	
		foreach my $srv (@{$self->{srv_list}})
		{
			$srv->print_results();
			print("\n");
		}
	}
}



###########################################
#
#	round function
#
###########################################
sub myround#($)
{
	my $res = $_[0];
	if($res <0){$res = int(100*$res - 0.5)/100 ;}
	else{$res = int(100*$res + 0.5)/100 ;}
	return $res;
}




####################################################
####################################################
####################################################
####################################################




1;
