#!/usr/bin/perl -w

######################################################
#
#	Example of wrapper exploration
#
#	This script explore impact of ambient temperature
#	and produce a normalized TCO and a normalized 
#	environmental impact.
#
#	The results are printed in a file <output_file_name>.txt
#	and a graph can be plot in a file <output_file_name>.eps
#
#	How to use it:
#	./ambient.pl	<dc config file>								datacenter configuration file
#					<resource config file>							resource configuration file
#					<output_file_name>								filename of the output file without extension (the extensions are .txt and .eps)
#
#
#	Example of command:
#		./ambient_temperature.pl template/dc_template.params template/s1_template.params ambient
#
######################################################

use strict;
use File::Copy;

use lib "../lib/"; 
use helper;

###########################################################
#
#	check valid input
#
###########################################################
my $argv_size = scalar @ARGV;
if ($argv_size < 3)
{
	print "usage: ./ambient.pl <dc config file> <resource config file> <output_file_name>\n";
	exit(0);
}


my $dc_filename=$ARGV[0];
my $resource_filename=$ARGV[1];

my $filename=$ARGV[2];



######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#get the current dir
my $currentdir=`pwd`;
chomp $currentdir;


#legende
my $legende="#temperature \t normalized TCO \t normalized env impact\n";

#open outputfile and print the legende
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;
print(OUTPUTFILE "$legende");

######################################################
#
# Specific Initialization
#
######################################################

my $tco_ref=0;
my $co2_ref=0;

my @temperature=("20","21","22","23","24","25","26","27","28","29","30");

my $temperature_ref=20;
my $pue_ref=2;

my $pue_coef= (2-1.65)/10;

#Acceleration factor
my $k=8.617e-05;
my $Ea=0.3;

######################################################
#
# The wrapper exploration start here
#
######################################################

my $ptr_dc=$helper->parserDCFile($dc_filename);
my %dc=%{$ptr_dc};
my $ptr_resource=$helper->parserResourceFile($resource_filename);
my %resource=%{$ptr_resource};

####################################################
# main exploration loop
####################################################

foreach my $temperature_current (@temperature)
{
	my $current_pue = $pue_ref - ($temperature_current - $temperature_ref) * $pue_coef ;
	
	my $AF=exp( ($Ea/$k) * ( (1/($temperature_ref+273)) -  (1/($temperature_current+273)) ) );
	
	my $current_mttf_coef = 1/$AF;
	
	#generate the current configuration files
	$dc{PUE}=$current_pue;
	$helper->createParameterFile("$currentdir/dc.params",\%dc);
	
	$resource{mttf_coef}=$current_mttf_coef;
	$helper->createParameterFile("$currentdir/s.params",\%resource);
	
	#call the eetco kernel and keep the output
	$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/$temperature_current.res");

	#get the TCO  & co2 values
	my $ptr_res=$helper->parserResultKernel("$currentdir/output/$temperature_current.res");
	my %res=%{$ptr_res};
	my $tco=$res{tco};
	my $co2=$res{co2peryr};
	
	if($temperature[0]==$temperature_current)#first iteration, keep the reference values to normalize
	{
		$tco_ref=$tco;
		$co2_ref=$co2;
	}
	
	my $normalized_tco=$tco/$tco_ref;
	
	my $normalized_co2 = $co2/$co2_ref;
	
	print(OUTPUTFILE "$temperature_current\t$normalized_tco\t$normalized_co2\n");
}

close(OUTPUTFILE);

#keep the output file in the current directory
copy("$currentdir/output/$filename.txt","$currentdir/$filename.txt") or die "Copy failed: $!";

##################################

####################################################
# gnulpot call
####################################################

my @curves_names=("Normalized TCO","Normalized CO2");
$helper->makeGnuplotCurves2D("$currentdir/$filename.txt","$currentdir","$filename","Temperature (in C)","Normalized (TCO or CO2)",\@curves_names);

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");


